import multiprocessing as mp
import time
import torch
import dgl
from numpy.testing import assert_array_equal

INTEGER = 2
STR = "hello world!"
HELLO_SERVICE_ID = 901231
TENSOR = torch.tensor([1.0, 2.0, 3.0])

class HelloResponse(dgl.distributed.Response):
    def __init__(self, hello_str, integer, tensor):
        self.hello_str = hello_str
        self.integer = integer
        self.tensor = tensor

    def __getstate__(self):
        return self.hello_str, self.integer, self.tensor

    def __setstate__(self, state):
        self.hello_str, self.integer, self.tensor = state


class HelloRequest(dgl.distributed.Request):
    def __init__(self, hello_str, integer, tensor, func):
        self.hello_str = hello_str
        self.integer = integer
        self.tensor = tensor
        self.func = func

    def __getstate__(self):
        return self.hello_str, self.integer, self.tensor, self.func

    def __setstate__(self, state):
        self.hello_str, self.integer, self.tensor, self.func = state

    def process_request(self, server_state):
        assert self.hello_str == STR
        assert self.integer == INTEGER
        new_tensor = self.func(self.tensor)
        res = HelloResponse(self.hello_str, self.integer, new_tensor)
        return res

def simple_func(tensor):
    return tensor
        

        

if __name__ == "__main__":
    dgl.distributed.connect_to_server(
        ip_config="rpc_ip_config_multithread.txt",
        num_servers=1,
    )
    dgl.distributed.register_service(
        HELLO_SERVICE_ID, HelloRequest, HelloResponse
    )
    req = HelloRequest(STR, INTEGER, TENSOR, simple_func)
    dgl.distributed.send_request(0, req)
    time.sleep(20)
    res0 = dgl.distributed.recv_response()
    res1 = dgl.distributed.recv_response()
    dgl.distributed.exit_client()